import { Model } from "starspot-core";
import JSONAPI from "./json-api";
/**
 * A Resource tells Starspot how to map models from your ORM into a JSON
 * response, and how to map incoming HTTP requests into changes to the
 * underlying database.
 *
 * Resources serve two purposes:
 *
 * * Static methods on the class tell Starspot how to find models in the
 *   database.
 * * Instances of Resource wrap an underlying model object and control how it
 *   is serialized to JSON and vice versa.
 *
 * For example, to find all instances of the Photo model, Starspot will call the
 * PhotoResource's static findAll() method, which should delegate that requests
 * to the database and return an array of model objects.
 *
 * Each of those returned models will be wrapped in a `PhotoResource` instance.
 * That instance will control how the underlying model is converted into JSON
 * used to respond to the request.
 */
declare class Resource<T extends Model> {
    model: T;
    _attributesList: string[];
    _relationshipsList: string[];
    ["constructor"]: typeof Resource;
    constructor(model?: T);
    validate(): Promise<boolean>;
    updateAttributes?(attributes: Resource.Attributes): Promise<void>;
    save?(): Promise<Model>;
    static create?(options: Resource.CreateOptions): Promise<Model>;
    static findByID?(id: JSONAPI.ID): Promise<Model>;
    static findAll?(): Promise<Model[]>;
}
export default Resource;
declare namespace Resource {
    interface CreateOptions {
        attributes: JSONAPI.AttributesObject;
    }
    interface Attributes {
        [attr: string]: any;
    }
}
export * from "./resource/decorators";
